/* agg - the news aggregator
 * Copyright (C) 2011 Andreas Waidler <arandes@programmers.at>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#define _XOPEN_SOURCE
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include "config.h"
#include "bool.h"
#include "fail.h"
#include "fs.h"

char   feed_title[TEXT_BUFFER_SIZE] = { 0 };
time_t feed_date = 0;
time_t feed_date_new = 0;
bool   feed_is_open = false;

char   item_title[TEXT_BUFFER_SIZE] = { 0 };
char   item_link [TEXT_BUFFER_SIZE] = { 0 };
char   item_desc [TEXT_BUFFER_SIZE] = { 0 };
time_t item_date = 0;

void sanitize(char* str)
{
	char* evil;
	while ((evil = strchr(str, '/'))) *evil = '\\';
}

void set_item_date(const char* date)
{
	struct tm t;
	assert(strptime(date, "%a, %d %b %Y %T %z", &t));
	item_date = mktime(&t);
	assert(item_date != -1);
	if (item_date <= feed_date) {
		printf("No new feeds assumed.\n");
		fail(0);
	} else if (item_date > feed_date_new) {
		feed_date_new = item_date;
	}
}

void set_item_link(const char* link)
{
	strncpy(item_link, link, TEXT_BUFFER_SIZE);
}

void set_item_desc(const char* desc)
{
	strncpy(item_desc, desc, TEXT_BUFFER_SIZE);
}

void set_item_title(const char* title)
{
	strncpy(item_title, title, TEXT_BUFFER_SIZE);
	sanitize(item_title);
}

void feed_open(const char* title)
{
	struct stat st;

	strncpy(feed_title, title, TEXT_BUFFER_SIZE);
	sanitize(feed_title);

	if (!stat(feed_title, &st)) feed_date = st.st_mtime;
	mkdir(feed_title, 0755);
	assert(!chdir(feed_title));
	feed_is_open = true;
}



void feed_flush()
{
	struct timeval times[2];
	times[0].tv_sec  = feed_date_new;
	times[0].tv_usec = 0;
	times[1].tv_sec  = feed_date_new;
	times[1].tv_usec = 0;

	if (!feed_is_open) return;

	assert(!chdir(".."));
	assert(!utimes(feed_title, times));
	feed_is_open = false;
}

void item_flush()
{
	struct timeval times[2];
	FILE* f;
	char buf[FILE_NAME_LENGTH + 1] = { 0 };
	assert(feed_is_open);

	if (item_title[0]) {
		strncpy(buf, item_title, FILE_NAME_LENGTH);
	} else {
		assert(item_desc[0]);
		strncpy(buf, item_desc, FILE_NAME_LENGTH);
	}

	assert(f = fopen(buf, "w"));
	if (item_title[0]) {
		fprintf(f, "%s\n\n", item_title);
	}
	if (item_desc[0]) {
		fprintf(f, "%s\n\n", item_desc);
	}
	if (item_link[0]) {
		fprintf(f, "Link: <a href=\"%s\">%s</a>\n", item_link, item_link);
	}
	fclose(f);

	/* FIXME: item date might not have been set. */
	times[0].tv_sec  = item_date;
	times[0].tv_usec = 0;
	times[1].tv_sec  = item_date;
	times[1].tv_usec = 0;

	assert(!utimes(buf, times));
}
