#!/bin/sh
# Copyright (C) 2011 Andreas Waidler <arandes@programmers.at>
#
#                 DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#       TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#
# 0. You just DO WHAT THE FUCK YOU WANT TO.

FEED="agg test feed"
ITEM1="Item 1"
ITEM2="Item 2"
ITEM3="Item 3"

function agg_run()
{
	cat "$1" | ./agg
	res=$?
	if [ $res -ne 0 ]; then
		echo "Failed to run agg ($res)."
		exit 1
	fi
}

function agg_fail()
{
	cat "$1" | ./agg
	res=$?
	if [ $res -eq 0 ]; then
		echo "Failed to fail running agg."
		exit 1
	fi
}

function agg_clean()
{
	cd "$FEED"
	find -type f -exec ../nomtime "rm '{}'" \;
	res=$?
	if [ $res -ne 0 ]; then
		echo "Failed to run nomtime."
		exit 1
	fi
	cd ..
}

function cleanup()
{
	rm -rf "$FEED" item "simple feed" "simple feed t" "simple feed d" "long feed" || exit 1
}

FAILURES=0
function t()
{
	printf "  %-32s: " "$1"
	$1
	result=$?
	if [ $result -ne 0 ]; then
		echo "FAIL"
		FAILURES=`expr $FAILURES + 1 `
	else
		echo "OK"
	fi
}

function test_feed_exists()  { [ -d "$1" ]; }
function test_feed_empty()   { [ `ls "$1" | wc -l` = "0" ]; }
function test_feed_date()    { [ `stat -c %Y "$1"` = "$2" ]; }
function test_item_exists()  { [ -f "$1" ]; }
function test_item_date()    { [ `stat -c %Y "$1"` = "$2" ]; }
function test_item_missing() { [ ! -e "$1" ]; }
# BUG: "`cat`" drops trailing \n.
function test_item_contents() { [ "`cat \"$1\"`" = "$2" ]; }

################## TEST CASES FOLLOW ######################

function test_simple_t_feed_exists() { test_feed_exists "simple feed t"; }
function test_simple_t_item_exists() { test_item_exists "simple feed t/item"; }
function test_simple_t_item_date()   { test_item_date "simple feed t/item" 0; }
function test_simple_t_item_contents() { test_item_contents "simple feed t/item" "item"; }

function test_simple_d_feed_exists() { test_feed_exists "simple feed d"; }
function test_simple_d_item_contents() { test_item_contents "simple feed d/item" "item"; }

LO=________________________________________________________________.
LC=________________________________________________________________
function test_long_feed_exists() { test_feed_exists "long feed"; }
function test_long_item_exists() { test_item_exists "long feed/$LC"; }
function test_long_item_contents { test_item_contents "long feed/$LC" "$LO"; }

function test_complete_feed_exists()  { test_feed_exists "$FEED"; }
function test_complete_feed_empty()   { test_feed_empty "$FEED"; }
function test_complete_feed_date()    { test_feed_date "$FEED" 1286052203; }
function test_complete_item1_exists() { test_item_exists "$FEED/$ITEM1"; }
function test_complete_item2_exists() { test_item_exists "$FEED/$ITEM2"; }
function test_complete_item3_exists() { test_item_exists "$FEED/$ITEM3"; }
function test_complete_item1_date()   { test_item_date "$FEED/$ITEM1" 1286052203; }
function test_complete_item2_date()   { test_item_date "$FEED/$ITEM2" 1270203000; }
function test_complete_item3_date()   { test_item_date "$FEED/$ITEM3" 1270119246; }
function test_complete_item1_contents()
{
	exp="\
Item 1

Random item.

Link: <a href=\"/dev/random\">/dev/random</a>"
	test_item_contents "$FEED/$ITEM1" "$exp"
}
function test_complete_item2_contents()
{
	exp="\
Item 2

Not so random item.

Link: <a href=\"/dev/urandom\">/dev/urandom</a>"
	test_item_contents "$FEED/$ITEM2" "$exp"
}
function test_complete_item3_contents()
{
	exp="\
Item 3

No item.

Link: <a href=\"/dev/null\">/dev/null</a>"
	test_item_contents "$FEED/$ITEM3" "$exp"
}

###############################################

echo "Cleaning directory..."
cleanup

echo "Running agg on feed without title..."
agg_fail tests_titleless.rss
t test_item_missing

echo "Running agg on feed with title after items..."
agg_fail tests_titlelate.rss
t test_item_missing

echo "Running agg on simple feed (titles)..."
agg_run tests_simple_titles.rss
t test_simple_t_feed_exists
t test_simple_t_item_exists
t test_simple_t_item_date
t test_simple_t_item_contents

echo "Running agg on simple feed (descriptions)..."
agg_run tests_simple_descriptions.rss
t test_simple_d_feed_exists
t test_simple_d_item_contents

echo "Running agg on missing directory..."
agg_run tests.rss
t test_complete_feed_exists
t test_complete_feed_date
t test_complete_item1_exists
t test_complete_item2_exists
t test_complete_item3_exists
t test_complete_item1_date
t test_complete_item2_date
t test_complete_item3_date
t test_complete_item1_contents
t test_complete_item2_contents
t test_complete_item3_contents

echo "Running agg on up-to-date directory..."
agg_run tests.rss 
t test_complete_feed_exists
t test_complete_feed_date
t test_complete_item1_exists
t test_complete_item2_exists
t test_complete_item3_exists
t test_complete_item1_date
t test_complete_item2_date
t test_complete_item3_date
t test_complete_item1_contents
t test_complete_item2_contents
t test_complete_item3_contents

echo "Deleting old news..."
agg_clean
t test_complete_feed_exists
t test_complete_feed_date
t test_complete_feed_empty

echo "Running agg on clean up-to-date directory..."
agg_run tests.rss 
t test_complete_feed_exists
t test_complete_feed_date
t test_complete_feed_empty

echo "Changing mtime..."
touch -md "1970-01-01 00:00:00.000000000 +0000" "$FEED"

echo "Running agg on clean outdated directory..."
agg_run tests.rss 
t test_complete_feed_exists
t test_complete_feed_date
t test_complete_item1_exists
t test_complete_item2_exists
t test_complete_item3_exists
t test_complete_item1_date
t test_complete_item2_date
t test_complete_item3_date
t test_complete_item1_contents
t test_complete_item2_contents
t test_complete_item3_contents

echo "Running agg on feed with long items..."
agg_run tests_long.rss
t test_long_feed_exists
t test_long_item_exists
t test_long_item_contents

echo "Cleaning up..."
cleanup

echo "Failed $FAILURES times!"
exit $FAILURES
